<?php
// LiveUser: A framework for authentication and authorization in PHP applications
// Copyright (C) 2002-2003 Markus Wolff
//
// This library is free software; you can redistribute it and/or
// modify it under the terms of the GNU Lesser General Public
// License as published by the Free Software Foundation; either
// version 2.1 of the License, or (at your option) any later version.
//
// This library is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
// Lesser General Public License for more details.
//
// You should have received a copy of the GNU Lesser General Public
// License along with this library; if not, write to the Free Software
// Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA

/**
 * DB_Complex permission administration class
 *
 * @package  LiveUser
 * @category authentication
 */

/**
 * Require the parent class definition
 */
require_once 'LiveUser/Admin/Perm/Container/DB_Medium.php';

/**
 * This is a PEAR::DB admin class for the LiveUser package.
 *
 * It takes care of managing the permission part of the package.
 *
 * A PEAR::DB connection object can be passed to the constructor to reuse an
 * existing connection. Alternatively, a DSN can be passed to open a new one.
 *
 * Requirements:
 * - Files "common.php", "Container/DB_Complex.php" in directory "Perm"
 * - Array of connection options must be passed to the constructor.
 *   Example: array("server" => "localhost", "user" => "root",
 *   "password" => "pwd", "database" => "AllMyPreciousData")
 *
 * @author  Christian Dickmann <dickmann@php.net>
 * @author  Markus Wolff <wolff@21st.de>
 * @author  Matt Scifo <mscifo@php.net>
 * @version $Id: DB_Complex.php,v 1.79 2004/10/23 08:51:45 arnaud Exp $
 * @package LiveUser
 */
class LiveUser_Admin_Perm_Container_DB_Complex extends LiveUser_Admin_Perm_Container_DB_Medium
{
    /**
     * Constructor
     *
     * @access protected
     * @param  mixed  Array or PEAR::DB object.
     * @param  array  configuration array not used atm
     * @return void
     */
    function LiveUser_Admin_Perm_Container_DB_Complex(&$connectOptions)
    {
        $this->LiveUser_Admin_Perm_Container_DB_Medium($connectOptions);
    }

    /**
     * Assign subgroup to parent group.
     *
     * First checks if groupId and subgroupId are the same then if
     * the child group does not have a parent group
     * already assigned to it. If so it returns an error object
     *
     * @access public
     * @param  integer id of parent group
     * @param  integer id of child group
     * @return mixed boolean, DB Error object or LiveUser Error Object
     */
    function assignSubgroup($groupId, $subgroupId)
    {
        if ($groupId == $subgroupId) {
            return LiveUser_Admin::raiseError(LIVEUSER_ERROR, null, null,
                'Parent group id is the same as the subgroup id');
         }

        $query = 'SELECT subgroup_id FROM
                  ' . $this->prefix . 'group_subgroups
                    WHERE
                      subgroup_id = ' . (int)$subgroupId;

        if (!is_null($this->dbc->getOne($query))) {
            return LiveUser_Admin::raiseError(LIVEUSER_ERROR, null, null,
                'Child group already has a parent group');
        }

        $query = 'SELECT group_id FROM 
                  ' . $this->prefix . 'group_subgroups
                    WHERE
                        subgroup_id = ' .  (int)$groupId . '
                      AND 
                        group_id = ' .  (int)$subgroupId;

        if (!is_null($this->dbc->getOne($query))) {
            return LiveUser_Admin::raiseError(LIVEUSER_ERROR, null, null,
                'This child group is already a Parent of the Parent group');
        }

        $query = 'INSERT INTO
                  ' . $this->prefix . 'group_subgroups
                  (group_id, subgroup_id)
                VALUES
                  (
                    ' . (int)$groupId . ',
                    ' . (int)$subgroupId . '
                  )';

        $result = $this->dbc->query($query);

        return $result;
    }

     /**
     * Unassign subgroup from parent group.
     *
     * Remove parent group from child group.
     *
     * @access public
     * @param  integer id of child group
     * @return mixed boolean, DB Error object or LiveUser Error Object
     */
    function unassignSubgroup($subgroupId)
    {
        $query = 'DELETE FROM
                  ' . $this->prefix . 'group_subgroups
                    WHERE
                      subgroup_id = ' . (int)$subgroupId;

        $result = $this->dbc->query($query);

        if (DB::isError($result)) {
            return $result;
        }

        return true;
    }

    /**
     * Deletes a group from the database
     *
     *
     * @access public
     * @param  integer id of deleted group
     * @param  boolean recursive delete of all subgroups
     * @return mixed   boolean or DB Error object
     */
    function removeGroup($groupId, $recursive = false)
    {
        // Recursive delete groups
        if ($recursive) {
            // Get all subgroups
            $query = 'SELECT
                      subgroup_id
                    FROM
                      ' . $this->prefix . 'group_subgroups
                    WHERE
                      groupd = ' . (int)$groupId;

            $result = $this->dbc->getCol($query);

            if (DB::isError($result)) {
                return $result;
            }

            // Recursive removeGroup() call for every subgroup
            foreach ($result as $subgroupId) {
                $res = $this->removeGroup($subgroupId, true);
                if (DB::isError($res)) {
                    return $res;
                }
            }
        }

        // Delete Subgroup assignments
        $query = 'DELETE FROM
                  ' . $this->prefix . 'group_subgroups
                WHERE
                  group_id = ' . (int)$groupId . ' OR
                  subgroup_id = ' . (int)$groupId;

        $result = $this->dbc->query($query);

        if (DB::isError($result)) {
            return $result;
        }

        parent::removegroup($groupId);

        return true;
    }

    /**
     * get parent group of group
     *
     * @access public
     * @param  integer id of subgroup
     * @return mixed integer or DB Error object
     */
    function getParentGroup($subgroupId)
    {
        $query = 'SELECT
                            group_id
                        FROM
                            ' . $this->prefix . 'group_subgroups
                       WHERE
                            subgroup_id = '
                                . (int)$subgroupId;

        $groupId = $this->dbc->getOne($query);

        return $groupId;
    }

    /**
     * Update implied status of right
     *
     * @access private
     * @param  integer id of right
     * @return mixed   boolean or DB Error object
     */
    function _updateImpliedStatus($rightId)
    {
        // Are there any implied rights?
        $query = 'SELECT
                  count(*)
                FROM
                  ' . $this->prefix . 'right_implied
                WHERE
                  right_id = ' . (int)$rightId;

        $count = $this->dbc->getOne($query);

        if (DB::isError($count)) {
            return $count;
        }

        $query = 'UPDATE
                  ' . $this->prefix . 'rights
                SET
                  has_implied = \'' . (((int)$count > 0) ? 'Y' : 'N') . '\'
                WHERE
                  right_id = ' . (int)$rightId;

        $result = $this->dbc->query($query);

        if (DB::isError($result)) {
            return $result;
        }

        // Job done ...
        return true;
    }

    /**
     * Imply right
     *
     * @access public
     * @param  integer id of right
     * @param  string  id of implied right
     * @return mixed   boolean or DB Error object
     */
    function implyRight($rightId, $impliedRightId)
    {
        // Imply Right
        $query = 'INSERT INTO
                  ' . $this->prefix . 'right_implied
                  (right_id, implied_right_id)
                VALUES
                  (
                    ' . (int) $rightId . ',
                    ' . (int) $impliedRightId . '
                  )';

        $result = $this->dbc->query($query);

        if (DB::isError($result)) {
            return $result;
        }

        // Update implied status
        return $this->_updateImpliedStatus($rightId);
    }

    /**
     * Unimply right
     *
     * @access public
     * @param  integer id of right
     * @param  string  id of implied right
     * @return mixed   boolean or DB Error object
     */
    function unimplyRight($rightId, $impliedRightId)
    {
        // Unimply right
        $query = 'DELETE FROM
                  ' . $this->prefix . 'right_implied
                WHERE
                  right_id         = ' . (int)$rightId . ' AND
                  implied_right_id = ' . (int)$impliedRightId;
        $result = $this->dbc->query($query);

        if (DB::isError($result)) {
            return $result;
        }

        // Update implied status
        return $this->_updateImpliedStatus($rightId);
    }

    /**
     * Overriden method to delete implied rights mapping
     * as well as the right.
     *
     * @access public
     * @param  int   right identifier
     * @return mixed true on success or DB error
     */
    function removeRight($rightId)
    {
        // delete from implied_rights unimply rights
        $query = 'DELETE FROM
                  ' . $this->prefix . 'right_implied
                WHERE
                  right_id         = ' . (int)$rightId;

        $result = $this->dbc->query($query);
		
        if (DB::isError($result)) {
            return $result;
        }

        parent::removeRight($rightId);

        // Update implied status
        return $this->_updateImpliedStatus($rightId);
    }

    /**
     * Get list of all groups
     *
     * This method accepts the following options...
     * <code>
     *  'where_user_id' = [PERM_USER_ID],
     *  'where_group_id' = [GROUP_ID],
     *  'where_owner_user_id' = [OWNER_USER_ID],
     *  'where_owner_group_id' = [OWNER_GROUP_ID],
     *  'where_is_active' = [BOOLEAN],
     *  'with_rights' = [BOOLEAN]
     * </code>
     *
     * @access public
     * @param  array  an array determining which fields and conditions to use
     * @param  boolean  determine whether or not to build a hierarchal result set
     * @return mixed array or DB Error object
     */
    function getGroups($options = null, $hierarchy = false)
    {
        if (!$this->init_ok) {
            return false;
        }

       $groups = parent::getGroups($options);

        if (DB::isError($groups)) {
            return $groups;
        }

        $query = 'SELECT
                      subgroups.group_id as group_id,
                      subgroups.subgroup_id as subgroup_id
                 FROM
                 ' . $this->prefix . 'group_subgroups subgroups';

        $subgroups = $this->dbc->getAll($query, null, DB_FETCHMODE_ASSOC);

        if (DB::isError($subgroups)) {
            return $subgroups;
        }

        // first it will make all subgroups, then it will update all subgroups to keep
        // everything up2date
        for ($i = 0; $i < 2; $i++) {
            foreach($subgroups as $subgroup) {
                if (isset($groups[$subgroup['group_id']])) {
                    $groups[$subgroup['group_id']]['subgroups'][$subgroup['subgroup_id']] = 
                        $groups[$subgroup['subgroup_id']];
                }
            }
        }
		
        if ($hierarchy) {
           foreach($subgroups as $subgroup) {
                if (isset($groups[$subgroup['subgroup_id']])) {
                   unset($groups[$subgroup['subgroup_id']]);
                }
            }
        }
		
        return $groups;
    }

    /**
     * Get list of all rights
     *
     * This method accepts the following options...
     *  'where_user_id' = [PERM_USER_ID],
     *  'where_group_id' = [GROUP_ID],
     *  'where_right_id' = [RIGHT_ID],
     *  'where_area_id' = [AREA_ID],
     *  'where_application_id' = [APPLICATION_ID],
     *  'with_areas' = [BOOLEAN],
     *  'with_applications' = [BOOLEAN]
     *  'with_inherited_rights' = [BOOLEAN]
     *  'with_implied_rights' = [BOOLEAN]
     *
     * @access public
     * @param  array    an array determining which fields and conditions to use
     * @param  boolean whether or not to build a hierarchal result set
     * @return mixed     array or DB Error object
     */
    function getRights($options = null, $hierarchy = false)
    {
        $query = 'SELECT
                  rights.right_id      AS right_id,
                  rights.area_id       AS area_id,
                  areas.application_id AS application_id,
                  rights.right_define_name AS define_name,
                  rights.has_implied       AS has_implied,
                  rights.has_level         AS has_level,
                  translations.name        AS name,
                  translations.description AS description';

        if (isset($options['where_user_id'])) {
            $query .= ', userrights.perm_user_id AS user_id';
            $query .= ', userrights.right_level AS right_level';
        } else if (isset($options['where_group_id'])
                && is_numeric($options['where_group_id'])) {
            $query .= ', grouprights.group_id AS group_id';
            $query .= ', grouprights.right_level AS right_level';
        }

        $query .= ' FROM
                  ' . $this->prefix . 'rights rights,
                  ' . $this->prefix . 'areas areas,
                  ' . $this->prefix . 'applications applications,';

        if (isset($options['where_user_id'])) {
            $query .= ' ' . $this->prefix . 'userrights userrights,';
        } else if (isset($options['where_group_id'])
                && is_numeric($options['where_group_id'])) {
            $query .= ' ' . $this->prefix . 'grouprights grouprights,';
        }

        $query .= ' ' . $this->prefix . 'translations translations
                WHERE';

        if (isset($options['where_right_id'])
                && is_numeric($options['where_right_id'])) {
            $query .= ' rights.right_id = '
                . (int)$options['where_right_id'] . ' AND';
        }

        if (isset($options['where_area_id'])
                && is_numeric($options['where_area_id'])) {
            $query .= ' rights.area_id = '
                . (int)$options['where_area_id'] . ' AND';
        }

        if (isset($options['where_application_id'])
                && is_numeric($options['where_application_id'])) {
            $query .= ' areas.application_id = '
                . (int)$options['where_application_id'] . ' AND';
        }

        if (isset($options['where_user_id'])) {
            $query .= ' userrights.perm_user_id = '
                . (int)$options['where_user_id'] . ' AND
                      userrights.right_id = rights.right_id AND';
        } else if (isset($options['where_group_id'])
                && is_numeric($options['where_group_id'])) {
            $query .= ' grouprights.' . $this->groupTableCols['required']['group_id']['name'] . ' = '
                . (int)$options['where_group_id'] . ' AND
                      grouprights.right_id = rights.right_id AND';
        }

        $query .= ' rights.area_id = areas.area_id AND
                  rights.right_id = translations.section_id AND
                  translations.section_type = ' . LIVEUSER_SECTION_RIGHT . ' AND
                  translations.language_id = '
                    . (int)($this->_langs[$this->getCurrentLanguage()]) . '
                GROUP BY
                  rights.right_id, rights.area_id, areas.application_id';

        if (isset($options['where_user_id'])) {
            $query .= ',userrights.perm_user_id';
            $query .= ',userrights.right_level';
        } else if (isset($options['where_group_id'])
                && is_numeric($options['where_group_id'])) {
            $query .= ',grouprights.group_id';
            $query .= ',grouprights.right_level';
        }

        $query .= ',rights.right_define_name, rights.has_implied,
                  rights.has_level, translations.name,
                  translations.description
                ORDER BY
                  rights.area_id ASC';

        $rights = $this->dbc->getAll($query, null, DB_FETCHMODE_ASSOC);

        if (DB::isError($rights)) {
            return $rights;
        }

        $_rights = array();
        if (is_array($rights)) {
            foreach ($rights as $key => $value) {
                $id = $value['right_id'];
                $_rights[$id] = $value;

                if (isset($options['with_areas'])) {
                    // Add area
                    $filter = array('where_area_id' => $value['area_id']);
                    $_rights[$id]['area'] =
                        array_shift($this->getAreas($filter));

                    if (DB::isError($_rights[$id]['area'])) {
                        return $_rights[$id]['area'];
                    }

                    if (isset($options['with_applications'])) {
                        // Add application
                        $filter = array('where_application_id' => $value['application_id']);
                        $_rights[$id]['application'] =
                            array_shift($this->getApplications($filter));

                        if (DB::isError($_rights[$id]['application'])) {
                            return $_rights[$id]['application'];
                        }
                    }
                }

                if (isset($options['with_implied_rights'])) {
                    $implied_rights = $this->getImpliedRights(
                        array(
                            'where_right_id' => $id,
                            'with_areas' => true,
                            'with_applications' => true
                        )
                    );

                    if (DB::isError($implied_rights)) {
                        return $implied_rights;
                    }

                    foreach($implied_rights as $right) {
                        if ($_rights[$right['right_id']]) {
                            continue;
                        }

                        $right['type'] = 'implied';

                        if ($hierarchy) {
                            $_rights[$id]['implied_rights'][$right['right_id']] = $right;
                            unset($_rights[$right['right_id']]);
                        } else {
                            $_rights[$right['right_id']] = $right;
                        }
                    }
                }

                if (!isset($_rights[$id]['type']) || !$_rights[$id]['type']) {
                    $_rights[$id]['type'] = 'granted';
                }
            }
        }

        if (isset($options['with_inherited_rights']) &&
            $options['with_inherited_rights'] &&
            (isset($options['where_user_id']) ||
            isset($options['where_group_id']))
        ) {
            $inherited_rights = $this->getInheritedRights($options);

            if (DB::isError($inherited_rights)) {
                return $inherited_rights;
            }

            foreach ($inherited_rights as $right) {
                if ($_rights[$right['right_id']]) {
                    continue;
                }

                $right['type'] = 'inherited';
                $_rights[$right['right_id']] = $right;
            }
        }

        return $_rights;
    }

    /**
     * Get implied rights
     *
     * @access public
     * @param  array    an array determining which fields and conditions to use
     * @return mixed    array or DB Error object
     */
    function getImpliedRights($options = array())
    {
        $query = 'SELECT
                      implied.right_id as right_id,
                      implied.implied_right_id as implied_right_id
                     FROM
                     ' . $this->prefix . 'right_implied implied';
        if (isset($options['where_right_id'])
                && is_numeric($options['where_right_id'])) {
            $query .= ' WHERE implied.right_id = ' . $options['where_right_id'];
        }

        $implied_rights = $this->dbc->getAll($query, null, DB_FETCHMODE_ASSOC);

        if (DB::isError($implied_rights)) {
            return $implied_rights;
        }

        $_rights = array();
        foreach($implied_rights as $implied) {
            $options['where_right_id'] = $implied['implied_right_id'];
            $implied_rights = $this->getRights($options);

            if (DB::isError($implied_rights)) {
                return $implied_rights;
            }

            $_rights = array_merge($_rights, $implied_rights);
        }

        return $_rights;
    }

    /**
     * Get inherited rights
     *
     * @access public
     * @param  array    an array determining which fields and conditions to use
     * @return mixed    array or DB Error object
     */
    function getInheritedRights($options = array())
    {
        if (isset($options['where_user_id'])) {
            $query = 'SELECT
                      groupusers.group_id as group_id
                     FROM
                     ' . $this->prefix . 'groupusers groupusers
                     WHERE groupusers.perm_user_id = ' . (int)$options['where_user_id'];
        } else {
            $query = 'SELECT
                      subgroups.group_id as group_id
                     FROM
                     ' . $this->prefix . 'group_subgroups subgroups
                     WHERE subgroups.subgroup_id = '
                        . $options['where_group_id'];
        }

        $groups = $this->dbc->getAll($query, null, DB_FETCHMODE_ASSOC);

        if (DB::isError($groups)) {
            return $groups;
        }

        $_rights = array();
        foreach($groups as $group) {
            $options['where_user_id'] = null;
            $options['where_group_id'] = $group['group_id'];
            $inherited_rights = $this->getRights($options);

            if (DB::isError($inherited_rights)) {
                return $inherited_rights;
            }

            $_rights = array_merge($_rights, $inherited_rights);
        }

        return $_rights;
    }

     /*
     * Delete a user.
     *
     * @access public
     * @param  string  perm_user_id
     * @return mixed   boolean or DB Error object
     */
    function removeUser($permId)
    {
        // sets owner_user_id to null
        $query = 'UPDATE
                  ' . $this->prefix . 'groups
                SET
                  owner_user_id = NULL
                WHERE
                  owner_user_id=' . $permId;

        $result = $this->dbc->query($query);

        if (DB::isError($result)) {
            return $result;
        }

        parent::removeUser($permId);

        return true;
    }

    /**
     * Update level status of right
     *
     * @access private
     * @param  integer id of right
     * @return mixed   boolean or DB Error object
     */
    function _updateLevelStatus($rightId)
    {
        // Are there any userrights with levels?
        $query = 'SELECT
                  count(*)
                FROM
                  ' . $this->prefix . 'userrights
                WHERE
                  right_id = ' . (int)$rightId . ' AND
                  right_level < '.LIVEUSER_MAX_LEVEL;

        $usercount = $this->dbc->getOne($query);

        if (DB::isError($usercount)) {
            return $usercount;
        }

        // Are there any grouprights with levels?
        $query = 'SELECT
                  count(*)
                FROM
                  ' . $this->prefix . 'grouprights
                WHERE
                  right_id = ' . (int)$rightId . ' AND
                  right_level < '.LIVEUSER_MAX_LEVEL;

        $groupcount = $this->dbc->getOne($query);

        if (DB::isError($groupcount)) {
            return $groupcount;
        }

        $count = $usercount + $groupcount;

        $query = 'UPDATE
                  ' . $this->prefix . 'rights
                SET
                  has_level = \'' . (((int)$count > 0) ? 'Y' : 'N') . '\'
                WHERE
                  right_id = ' . (int)$rightId;

        $result = $this->dbc->query($query);

        if (DB::isError($result)) {
            return $result;
        }

        // Job done ...
        return true;
    }

    /**
     * Grant right to user
     *
     * @access public
     * @param  string  id of user
     * @param  integer id of right
     * @param  integer level of right (can be negative to revoke/lower a right level)
     * @return mixed   boolean or DB Error object
     */
    function grantUserRight($permId, $rightId, $right_level = LIVEUSER_MAX_LEVEL)
    {
        //return if this user already has right
        $query = 'SELECT
                  count(*)
                FROM
                  ' . $this->prefix . 'userrights
                WHERE
                  perm_user_id  = ' . $permId . ' AND
                  right_id      = ' . (int)$rightId;

        $count = $this->dbc->getOne($query);

        if (DB::isError($count) || $count != 0) {
            return false;
        }

        $query = 'INSERT INTO
                  ' . $this->prefix . 'userrights
                  (perm_user_id, right_id, right_level)
                VALUES
                  (
                    ' . $permId . ',
                    ' . (int) $rightId . ',
                    ' . (int) $right_level . '
                  )';

        $result = $this->dbc->query($query);

        if (DB::isError($result)) {
            return $result;
        }

        $this->_updateLevelStatus($rightId);

        // Job done ...
        return true;
    }

    /**
     * Grant right to group
     *
     * @access public
     * @param  integer id of group
     * @param  integer id of right
     * @param  integer right level
     * @return mixed   boolean or DB Error object
     */
    function grantGroupRight($groupId, $rightId, $right_level = LIVEUSER_MAX_LEVEL)
    {
        //return if this group already has right
        $query = 'SELECT
                  count(*)
                FROM
                  ' . $this->prefix . 'grouprights
                WHERE
                  ' . $this->groupTableCols['required']['group_id']['name'] . ' = '
                    . (int)$groupId . ' AND
                  right_id = ' . (int)$rightId;

        $count = $this->dbc->getOne($query);

        if (DB::isError($count) || $count != 0) {
            return false;
        }

        $query = 'INSERT INTO
                  ' . $this->prefix . 'grouprights
                  (' . $this->groupTableCols['required']['group_id']['name'] . ', right_id, right_level)
                VALUES
                  (
                    ' . (int)$groupId . ',
                    ' . (int)$rightId . ',
                    ' . (int)$right_level . '
                  )';

        $result = $this->dbc->query($query);

        if (DB::isError($result)) {
            return $result;
        }

        $this->_updateLevelStatus($rightId);

        // Job done ...
        return true;
    }
}
?>